export class ArrowCurve
{
	constructor(arrowHeadSprite, arrowNodeSprite, numOfArrowNodes, runtime)
	{
		this.node = arrowNodeSprite;
		this.head = arrowHeadSprite;
		this.nodeNum = numOfArrowNodes;
		this.scaleFactor = 1;
		
		this.origin = {};
		this.nodeList = [];
		this.controlPoints = [];
		this.controlPointfactors = [
			{x:-0.3, y: 0.8},
			{x:0.1, y:1.4}
		];
		
		this.rt = runtime;
	}
	
	initialize(originX, originY)
	{
		//create the arrow node emittter point
		this.origin = {x:originX, y:originY};
		
		//create all the arrow nodes and head
		for (let i = 0; i < this.nodeNum; i++) {
		  const node = this.node.createInstance("Arrow", originX, originY, false);
		  this.nodeList.push(node);
		}
		
		//create head
		const head = this.head.createInstance("Arrow", originX, originY, false);
		this.nodeList.push(head);
		
		//create control points
		for (let i = 0; i < 4; ++i) {
		  this.controlPoints.push({x:0, y:0});
		}
	}
	
	tick(targetX, targetY)
	{
		//p0 is at the arrow emitter point (origin)
		this.controlPoints[0] = {x:this.origin.x, y:this.origin.x};
		
		//p3 is at the mouse mouse potion
		this.controlPoints[3] = {x:targetX, y:targetY};
		
		//p1, p2 are determined by p0,p3
		//p1 = p0 + (p3 - p0) * v2(-0.3, 0.8)
		//p2 = p0 + (p3 - p0) * v2(-0.1, 1.4)	
		this.controlPoints[1] = this._calculateControlPoint(this.controlPoints[0], this.controlPoints[3], this.controlPointfactors[0]);
		this.controlPoints[2] = this._calculateControlPoint(this.controlPoints[0], this.controlPoints[3], this.controlPointfactors[1]);

		//calculate node postions
		for(let i = 0; i < this.nodeList.length; i++)
		{	
			//console.log(i, this.nodeList[i]);
			
			//used to get the position along the bezier curve		
			const t = Math.log2(1 * i / (this.nodeList.length-1) + 1);
			
			//cubic bezier curve
			//B(t) = (1-t)^3 * p0 +3 * (1-t)^2 * t * p1 +3 * (1-t) * t^2 * p2 + t^3 * p3
			const x = this._calculateCubicBezierCurveX(t, this.controlPoints[0], this.controlPoints[1], this.controlPoints[2], this.controlPoints[3]);
			const y = this._calculateCubicBezierCurveY(t, this.controlPoints[0], this.controlPoints[1], this.controlPoints[2], this.controlPoints[3]);
						 
			//set node position
			this.nodeList[i].x = x;
			this.nodeList[i].y = y;
			
			//calculate rotation
			if(i>0)
			{
				this.nodeList[i].angle = C3.angleTo(this.nodeList[i].x, this.nodeList[i].y, this.nodeList[i-1].x, this.nodeList[i-1].y); 
			}
			
			//calculate scale
			const scale = this.scaleFactor * (1 - 0.03 * (this.nodeList.length-1-i));
			this.rt.callFunction("setScale", this.nodeList[i].uid, scale);
		}
	}
	
	
	_calculateCubicBezierCurveX(t, p0, p1, p2, p3)
	{
		const t1 = p0.x * Math.pow(1-t, 3);
		const t2 = p1.x * 3 * Math.pow(1-t, 2) * t;
		const t3 = p2.x * 3 * (1-t) * Math.pow(t, 2);
		const t4 = p3.x * Math.pow(t, 3)
		
		return t1+t2+t3+t4;
	}
	
	_calculateCubicBezierCurveY(t, p0, p1, p2, p3)
	{
		const t1 = p0.y * Math.pow(1-t, 3);
		const t2 = p1.y * 3 * Math.pow(1-t, 2) * t;
		const t3 = p2.y * 3 * (1-t) * Math.pow(t, 2);
		const t4 = p3.y * Math.pow(t, 3)
		
		return t1+t2+t3+t4;
	}
	
	_calculateControlPoint(p0, p3, factor)
	{
		 const x = p0.x + (p3.x - p0.x) * factor.x;
		 const y = p0.y + (p3.y - p0.y) * factor.y;
		 return {x:x, y:y};
	}
}



/*

// Part of a class, a, b, c, d are the four control points of the curve
x: function (t) {
    return ((1 - t) * (1 - t) * (1 - t)) * this.a.x
           + 3 * ((1 - t) * (1 - t)) * t * this.b.x
           + 3 * (1 - t) * (t * t) * this.c.x
           + (t * t * t) * this.d.x;
},

y: function (t) {
    return ((1 - t) * (1 - t) * (1 - t)) * this.a.y
           + 3 * ((1 - t) * (1 - t)) * t * this.b.y
           + 3 * (1 - t) * (t * t) * this.c.y
           + (t * t * t) * this.d.y;
}


================


function Bezier(a, b, c, d) {
    this.a = a;
    this.b = b;
    this.c = c;
    this.d = d;

    this.len = 100;
    this.arcLengths = new Array(this.len + 1);
    this.arcLengths[0] = 0;

    var ox = this.x(0), oy = this.y(0), clen = 0;
    for(var i = 1; i <= this.len; i += 1) {
        var x = this.x(i * 0.05), y = this.y(i * 0.05);
        var dx = ox - x, dy = oy - y;        
        clen += Math.sqrt(dx * dx + dy * dy);
        this.arcLengths[i] = clen;
        ox = x, oy = y;
    }
    this.length = clen;    
}

https://gamedev.stackexchange.com/questions/5373/moving-ships-between-two-planets-along-a-bezier-missing-some-equations-for-acce/5427#5427

Bezier.prototype = {
    map: function(u) {
        var targetLength = u * this.arcLengths[this.len];
        var low = 0, high = this.len, index = 0;
        while (low < high) {
            index = low + (((high - low) / 2) | 0);
            if (this.arcLengths[index] < targetLength) {
                low = index + 1;

            } else {
                high = index;
            }
        }
        if (this.arcLengths[index] > targetLength) {
            index--;
        }

        var lengthBefore = this.arcLengths[index];
        if (lengthBefore === targetLength) {
            return index / this.len;

        } else {
            return (index + (targetLength - lengthBefore) / (this.arcLengths[index + 1] - lengthBefore)) / this.len;
        }
    },

    mx: function (u) {
        return this.x(this.map(u));
    },

    my: function (u) {
        return this.y(this.map(u));
    },
	*/
